package HSPC::RichEditor;
## wrapper for WYSIWYG editor [FCKeditor 2.0, http://www.fckeditor.net]
## please ask ndedik@sw.ru on problems and feature requests!

use strict;

use HSPC::Config;

use HSPC::Localization;

use HSPC::Util::String qw(sw_escape_for_code);
use HSPC::WebSystemLib;
use SWsoft::protect; ## mark this module for encryption upon build

use HSPC::MT::PM; ## site files details storage
use HSPC::WebUpload; ## file uploading abilites

use XML::Simple;

## This function sends the results of the uploading process.
## => error (error number)
## => url
## => name
## => msg (optional custom message)
sub _send_results {
	my %h = (
		error => '0',
		url => '',
		name => '',
		msg => '',
		@_
	);
	my $content = '<script type="text/javascript">window.parent.' . 
		(sw_uriparam('a') eq 'browse' ? 'frames[\'frmUpload\'].' : '') .
		'OnUploadCompleted(' . $h{error} . ', ' . join(',',
			map {
				'"' . sw_escape_for_code(string => $_) . '"'
			} ($h{url}, $h{name}, $h{msg})) .
		')</script>';
	return {STATUS => 'CUSTOM', CONTENT => "Content-Type: text/html;\n\n$content"};
}

## Performs operations for browsing site files in FCKeditor browser (browser connector).
## => site (current site object)
## => q_get (GET method query)
sub _browse {
	my %h = (@_);

	my $connector = {
		command => sw_argparam('Command') || '',
		resourceType => sw_argparam('Type') || '',
		CurrentFolder => {
			path => $h{q_get}->{CurrentFolder},
			url => $CONFIG{HSPCROOT_DIR_DATAUPLOAD} . $h{q_get}->{CurrentFolder},
		},
	};
	if (sw_argparam('Command') =~ /^GetFolders(AndFiles)?$/) {
		my ($files_show, $folder) = ($1, $h{q_get}->{CurrentFolder});
		## prepare and execute request for site files list
		my $request = HSPC::DataRequest->new(cols => [
			{code => 'id'},
			{code => 'name'},
			{code => 'path', for_where => 1, value => '/data' . $folder},
			{code => 'size'},
			{code => 'is_image'},
			{code => 'site_id', for_where => 1, value => $h{site}->id},
		]);
		HSPC::MT::PM->find_site_files(request => $request);
		foreach (@{$request->{data}}) {
			$_->[2] =~ s/^\Q$CONFIG{HSPCROOT_DIR_DATAUPLOAD}\E//; ## remove data dir from path
		}
		## add data to XML response
##?	!!! code reserved for unlimited folder navigation, currently disabled to site files DB limitations
##?		my $folder = $CONFIG{HSPCROOT_ROOT} . $CONFIG{HSPCROOT_DIR_DATAUPLOAD} . $h{q_get}->{CurrentFolder};
##?		opendir DIR, $folder;
##?		$connector->{Folders} = {
##?			Folder => [
##?				map {
##?					{
##?						name => $_
##?					}
##?				} grep {-d $folder . $_ && !/^\./} readdir DIR
##?			]
##?		};
##?		closedir DIR;
		my %seen; ## avoid duplicate folder names
		$connector->{Folders} = {
			Folder => [
				map {
					{
						name => $_->[2],
					}
				} grep {$_->[2] =~ s/^\Q$folder\E([^\/]+).+$/$1/ && !$seen{$_->[2]}++}
					@{$request->{data}}
			]
		};
		$connector->{Files} = {
			File => [
				map {
					{
						name => $_->[1],
						size => int(($_->[3] - $_->[3] % 1024) / 1024),
					}
				} grep {$_->[2] eq $folder && (sw_uriparam('t') eq 'img'
					|| sw_argparam('Type') eq 'Image' ? $_->[4] : 1)}
					@{$request->{data}}
			]
		} if ($files_show);
	} elsif (sw_argparam('Command') eq 'CreateFolder') {
##?	!!! code reserved for folder creation, currently disabled due to site files DB limitations
##?		my $folder = $CONFIG{HSPCROOT_ROOT} . $CONFIG{HSPCROOT_DIR_DATAUPLOAD} .
##?			$h{q_get}->{CurrentFolder} . $h{q_get}->{NewFolderName};
##?		if (-d $folder) {
##?			$connector->{Error} = {number => 101};
##?		} elsif ($h{q_get}->{NewFolderName} !~ /^([\d\w-]+)$/ || !mkdir($folder, 0777)) {
##?			$connector->{Error} = {number => (!$1 ? 102 : 103)};
##?		} else {
##?			$connector->{Error} = {number => 0};
##?		}
		$connector->{Error} = {number => 103}; ## folder creation forbidden
	}
	my $response =
		XMLout({Connector => $connector},
			xmldecl => '<?xml version="1.0" encoding="utf-8"?>',
			keeproot => 1, ## no unneeded <opt> tag
			keyattr => [],
		);
	return {STATUS => 'CUSTOM', CONTENT =>
		"Cache-Control: no-cache;\nPragma: no-cache;\nExpires: -1;\nContent-Type: text/xml;\n\n$response"};
}

## Uploads files on request from FCKeditor (upload connector).
## => site (current site object)
## => q_get (GET method query)
sub _upload {
	my %h = (@_);

	## check file type
	my $filename = $ENV{q}->can('upload')
		? $ENV{q}->upload('NewFile')->filename
		: sw_argparam('NewFile');
	return _send_results(error => 202)
		if sw_uriparam('t') eq 'img' && $filename !~ /\.(gif|jpe?g|png)$/i;

	my($path) = $CONFIG{HSPCROOT_DIR_DATAUPLOAD} . $h{q_get}->{CurrentFolder};
	my $upload = HSPC::WebUpload::upload_file(
		view_name		=> 'NewFile',
		base_path		=> $CONFIG{HSPCROOT_ROOT},
		new_rel_path	=> $path,
	);				

	## check upload results
	return _send_results(error => 202, msg => $upload ? $upload : string('please_select_file_for_uploading'))
		unless (ref($upload) eq 'HASH');
	
	my $file = HSPC::MT::PM->new_site_file();
	$file->upload($upload);
	$file->site_id($h{site}->id);
	$file->image_alt($file->title);
	$file->save();
	$file->owner_id($ENV{security_obj}->account_no);

	return _send_results(error => ($upload->{rel_path} eq $path ? 0 : 201),
		url => $upload->{rel_path} . $upload->{file_name},
		name => $upload->{rel_path} . $upload->{file_name});
}

sub teaser {
	my %h = (
		site	=> HSPC::MT::PM->find_site(owner_id => $ENV{security_obj}->account_no, need => 1),
		q_get	=> {map {split(/=/, $_)} split(/&/, $ENV{'QUERY_STRING'})}, ## no need to unescape (plain text)
	);
	$h{q_get}->{CurrentFolder} =~ s/\.\.\/|\/\///g; ## for security purposes
	$h{q_get}->{CurrentFolder} .= '/' unless ($h{q_get}->{CurrentFolder} =~ /\/$/);

	if (sw_uriparam('a') eq 'upload' || $h{q_get}->{Command} eq 'FileUpload') {
		return _upload(site => $h{site}, q_get => $h{q_get});
	} elsif (sw_uriparam('a') eq 'browse') {
		return _browse(site => $h{site}, q_get => $h{q_get});
	}

	return {STATUS => 'ERROR'};
}

1;
