package HSPC::Transaction;

use strict;
use SWsoft::protect;	# Mark this module for encryption upon build


use HSPC::Logger qw(sw_log_debug sw_die);
use HSPC::WebDB qw(trans);

##--------------------------------------------------
## Module variables
##-------------------------------------------------- 
my $level = 0;
my $tid = 0;

## XXX TODO kmv:
## 	if fork will be executed during open transaction,
##  child will try to roll it back on destructor,
##  and this will cause sw_die. Now this problem is
##  not actual, but we should keep it in mind.

##--------------------------------------------------
## Usage: HSPC::Transaction->new(comment=>"start order saving")
## Description:	constructor, should be used only in trans.
## Parameters:
##     comment => comment
##     dbname  => db name, MAIN by default
## Return value: object
sub new {
	my $class = shift;
	my %h = (
		comment => undef, ## comment
		dbname  => undef, ## db name
		webdb   => undef, ## webdb object.
		@_
	);
	$h{webdb} ||= new HSPC::WebDB(dbname => $h{dbname});

	## open transaction
	my $err = $h{webdb}->select_open();
	sw_die("select_open returns error: $err") if $err;

	my $self = {
		finished	=> 0,
		tid		=> ++$tid,
		level		=> ++$level,

		map { $_ => $h{$_} } qw/comment webdb/
	};
	bless $self, $class;

	## log message
	$self->__logstr("open");

	$self;
};

##--------------------------------------------------
## Usage:  $trans->commit();
## Description: commit transaction.
## Parameters:	no parameters
## Return value: undef
sub commit {
	my $self = shift;

	sw_die("second commit") if $self->{finished};
	
	$self->{finished} = 1;
	$level--;
	
	$self->__logstr("commit");
	
	my $err = $self->{webdb}->select_commit();	
	sw_die("select_commit returns error: $err") if $err;

	return undef;
};

##--------------------------------------------------
## Usage:  $trans->rollback();
## Description: commit transaction.
## Parameters:	no parameters
## Return value: undef
sub rollback { $_[0]->__rollback( logstr => "rollback" ) }

##--------------------------------------------------
## destructor
sub DESTROY {
	my $self = shift;

	$self->__rollback( logstr => "autorollback" )
		unless $self->{finished};
}

##--------------------------------------------------
## make log message
sub __logstr {
	my $self = shift;
	my $action = shift;

	my $tid = sprintf("%02d", $self->{tid});
	my $lev = sprintf("%02d", $self->{level});

	HSPC::Logger->new()->debug(
		"TRANSACTION #$tid [$lev] $action $self->{comment}");
}

##--------------------------------------------------
## rollback transaction
## Parameters: no parameters
## no_log => 
## > undef
sub __rollback {
	my $self = shift;
	my %h = (logstr => 'no reason', @_);

	sw_die("second rollback") if $self->{finished};

	$self->{finished} = 1;
	$level--;

	$self->__logstr($h{logstr});

	my $err = $self->{webdb}->select_rollback();
	sw_die("select_rollback returns error: $err") if $err;

	return undef;
}

1;
__END__
