package HSPC::WebSystemLib;
use strict;

our @EXPORT = qw(
	&sw_layout_redirect_local &sw_layout_redirect &sw_layout_error &sw_layout_error_app
	&sw_layout_error404 &sw_layout_error503 &sw_layout_forbidden &sw_layout_parse_template
	&sw_layout_template_path &sw_sorted_lang_options
	&sw_web_forbidden &sw_web_status_503 &sw_web_not_found
	&sw_web_http_header &sw_web_html_header &sw_web_html_footer &sw_web_redirect
	&sw_web_redirect_local &sw_web_parse_template &sw_web_parse_text_template &sw_uriparam &sw_param &sw_uriparams
	&sw_argparam &sw_argparam_geo &sw_argparam_datetime &sw_argparam_period
	&sw_argparams_obj &sw_web_server_protocol &sw_full_name &sw_argparam_list
	&sw_sec_env
	&sw_web_server_name &sw_web_server_port &sw_provider_def_lang
	&sw_web_prepare4browser &sw_fill_lang_in_list &sw_web_redirect_comprep
	&sw_secure_link
);
use base 'Exporter';

use CGI;
use Text::Template;
use Params::Validate qw(:all);

use SWsoft::protect;	# Mark this module for encryption upon build

use HSPC::Config;
use HSPC::Logger qw(sw_atrace sw_log_warn sw_log_error sw_die);
use HSPC::Localization;
use HSPC::Localization::Date qw(time_local_to_gmt);
use HSPC::Util::String qw(sw_trim);

# > local_url
sub sw_layout_redirect_local{
	my $url = shift;
	my $self = $ENV{layout};

	sw_die('[WEBSYSTEM] (ERROR) undefined param in sw_layout_redirect_local()')
		unless defined $url;

	## if call without layout object
	return sw_web_redirect( $url )
		unless ($self);

	$self->{redirect_local} = $url;
	$self->{status} = 'REDIRECT';

	return {STATUS=>'REDIRECT'};
}

# > local_url
sub sw_layout_redirect{
	my $url = shift;
	my $self = $ENV{layout};

	sw_die('[WEBSYSTEM] (ERROR) undefined param in sw_layout_redirect()')
		unless defined $url;

	$self->{redirect} = $url;
	$self->{status} = 'REDIRECT';

	return {STATUS=>'REDIRECT'};
}


# > message
sub sw_layout_error{
	my $error = shift;

	$ENV{q}->can('error')
		? $ENV{q}->error($error)
		: $ENV{layout}->set_error($error, $error);
	return {STATUS=>'ERROR'};
}

# > message
sub sw_layout_error_app{
	my $c = shift;
	warn $c if $c;

	$ENV{q}->can('error')
		? $ENV{q}->error('ERROR')
		: $ENV{layout}->set_error_app;
	return {STATUS=>'ERROR'};
}

# > warn_message # REQ_N #
sub sw_layout_error404{
	my $c = shift;
	warn $c if $c;

	$ENV{q}->can('error')
		? $ENV{q}->error('404')
		: $ENV{layout}->set_error_404;
	return {STATUS=>'ERROR'};
}


# > warn_message # REQ_N #
sub sw_layout_error503{
	my $c = shift;
	warn $c if $c;

	$ENV{q}->can('error')
		? $ENV{q}->error('503')
		: $ENV{layout}->set_error_503;
	return {STATUS=>'ERROR'};
}

sub sw_layout_forbidden{
	my $c = shift;

	$ENV{q}->can('error')
		? $ENV{q}->error('403')
		: $ENV{layout}->set_error_forbidden;
}

# > object_class
# > template_name
# > hash
sub sw_layout_parse_template{
	my $class = shift;
	my $template = shift;
	my $hash = shift;

	my $trace = sw_atrace("Template " . $template);

	my $template_path = sw_layout_template_path($class,$template);

	return sw_web_parse_template($template_path,undef,$hash);
}

# > object_class
# > template_name
sub sw_layout_template_path{
	my $class = shift;
	my $template_name = shift;

	my $self = $ENV{layout};
	$class =~ s#::#/#g;
	my $template_path = $ENV{system_obj}->find_text_template_path(tmpl_path=>"site_$self->{site_id}".'/'.$class.'/'.$template_name);

	return $template_path;
}

sub sw_web_forbidden {
	if ($ENV{q}->can('error')) {
		$ENV{q}->error('403');
		exit;
	}

	if( defined $ENV{layout} ){
		$ENV{layout}->set_error_forbidden();
		exit;
	}
	exit 403;
}

sub sw_web_status_503 {
	if ($ENV{q}->can('error')) {
		$ENV{q}->error('503');
		exit;
	}

	if( defined $ENV{layout} ){
		$ENV{layout}->set_error_503();
		exit;
	}

	exit 503;
}

sub sw_web_not_found {
	if ($ENV{q}->can('error')) {
		$ENV{q}->error('404');
		exit;
	}

	if( defined $ENV{layout} ){
		$ENV{layout}->set_error_404();
		exit;
	}

	exit 404;
}


# pring HTTP request header
# => NO_CACHE  : disable page caching
sub sw_web_http_header{
	my %h = (
		type		=> 'text/html',
		NO_CACHE	=> 1,
		filename	=> undef,
		errorcode	=> undef,
		@_
	);

	my $cookies = $ENV{cookies_out};
	my $charset = $LOCALE{local_charset} || 'UTF-8';

	my %headers = (
		-type => $h{type},
		-charset => $charset,
		-cookie => %$cookies ? [values %$cookies] : undef,
	);

	if ( $h{filename} ) {
		$headers{'-Content_Disposition'} = 'attachment; filename="'.$h{filename}.'"';
	}

	if ( $h{errorcode} ) {
		$headers{'-status'} = $h{errorcode};
	}

	if ($h{NO_CACHE}) {
		##$headers{-expires} = 'Thu Jan  1 00:00:00 1970';
		##$headers{-expires} = '+10m';
		$headers{-expires} = '-1d';
		##$headers{'Cache-Control'} = "";
		##$headers{'pragma'} = "";
		##$q->cache(1);
	}

	my ($header) = CGI::header(%headers, '-nph' => 1) =~ /^[^\n]+\n(.+)$/s;
	print $header;
}

# return HTML page header
#	=> page_title : Page Browser Title
#	=> page_head : any code, will be posted in <head>....</head> HTML
#   => include_favicon : =1 for including fav_icon
#	=> no_body : do not post <body> tag
#	< result
sub sw_web_html_header{
	my %h=(
		charset=>$LOCALE{local_charset}||'UTF-8',
		skin_id=>$ENV{system_obj}->ui_skin?$ENV{system_obj}->ui_skin->id:undef,
		skin_version=>$ENV{system_obj}->ui_skin_version,
		base_url => &sw_web_server_protocol().'://'.&sw_web_server_name,
    @_);

    return sw_web_parse_template(__PACKAGE__,'web_header.tmpl',\%h);
}

# return HTML page footer
sub sw_web_html_footer{
	return sw_web_parse_template(__PACKAGE__,'web_footer.tmpl');
}

# > url
sub sw_web_redirect{
	my $url = shift;

	if(!$url){
		sw_die('[WEBSYSTEM] (ERROR) url undefined in sw_web_redirect()');
		exit;
	}
	sw_die('[WEBSYSTEM] (ERROR) url containts "\n" symbol that may causes apache child down')
		if $url =~ /\n/;

	#### old non-layout code supporting ###
	if( ! $ENV{layout}  ){
		$ENV{_redirect_ok}=1;
	}
	#### /old non-layout code supporting ###

	## redirect to referres screen, if it found in comprep and browsing history
	my ($node, $tool, $uri, $uri_params);
	($uri = $url) =~ s#^http(s)?://([^/])+##;
	(undef, undef, $tool, $uri, $uri_params) = split('/', $uri, 5);
	if ($ENV{COMPREP}) {
		## lookup node
		if ($uri =~ /^\d\d(?:\.\d{2,3})*$/) {
			$node = $ENV{COMPREP}->node(uri => $uri);
		} elsif ($uri =~ /^[\w-]+$/) {
			$node = $ENV{COMPREP}->node(alias => $uri);
		}
		if ($node && $node->{referrers}){
			my @refs = split ',', $node->{referrers};
			for (my $index = $#{$ENV{ session }->{aliases_history}}; $index >= 0; $index--){
				foreach my $ref (@refs){
					if (${$ENV{ session }->{aliases_history}}[$index]->{alias} eq $ref){
					    my $uri_status;
						if ($uri_params =~ /\/(sl(?:e|x|xe)?),([^\/]+)/ ){
							$uri_status = "/$1,$2";
						}
						$url = $ENV{COMPREP}->link(alias => $ref)
						. $uri_status
						. '/' . ${$ENV{ session }->{aliases_history}}[$index]->{uri_params};
					}
				}
		    }
		}
	}

	my $redirect_header = $ENV{q}->redirect(-uri => $url, -cookie => [values %{$ENV{cookies_out}}]);
	print $redirect_header
		if $ENV{layout};
	exit;
}

sub sw_web_redirect_comprep {
	sw_web_redirect($ENV{COMPREP}->link(@_));
}

## Test security for node according to 'alias' and either show link with 'text' or just 'text'.
sub sw_secure_link {
	validate(@_, {
		text		=> ## link description
					{ type => SCALAR },
		target		=> ## link target window
					{ type => SCALAR, optional => 1 },
		alias		=> ## node alias
					{ type => SCALAR },
		uri_params	=> ## URI params, just as in HSPC::CompRep::link
					{ type => ARRAYREF | HASHREF, optional => 1 },
	});
	my %h = @_;

	my $node = $ENV{COMPREP}->node(alias => $h{alias});
	my $link = $ENV{COMPREP}->link(alias => $h{alias}, $h{uri_params})
		if $node
			&& !$node->{filtered_out}
			&& $ENV{security_obj}->test_security($node->{alias});

	my $target = qq|target="$h{target}"|
		if $h{target};
	return $link ? qq|<a $target href="$link">$h{text}</a>| : $h{text};
}

# =>> url
# =>  status_id
# =>  error_id
sub sw_web_redirect_local{
	my %h=(@_);
	sw_die('url is required for sw_web_redirect_local') unless(defined($h{url}));
	$h{url} =~ s!\n|\r!!g;
	my $url = &sw_web_server_protocol . '://' . &sw_web_server_name . $h{url};
	$url.=qq{/sl,$h{status_id}} if $h{status_id};
	$url.=qq{/sle,$h{error_id}} if $h{error_id};

	sw_web_redirect($url);
}

# > template_path/class_name/object
# > template_name
# > input
# >= named args for Text::Template->fill_in()
sub sw_web_parse_template{
	my $template_path = shift;
	my $template_name = shift;
	my $input = shift;
	my %args=(@_);
	my $sys = $ENV{HSPC_SYSTEM};

	## pass skin object to template
	require HSPC::MT::CU;
	my $skin = $ENV{skin_obj} || HSPC::MT::CU->find_default_skin();
	$input->{skin} = \$skin;

	if($template_path=~m|\:\:|g){
		$template_path =~ s#::#/#g;
	};
	$template_path .= ($template_path ? '/' : '').$template_name
		if defined $template_name;

	$template_path = $sys->find_text_template_path(tmpl_path=>$template_path);

	if (! -r $template_path) {
		my $error = "[WEBSYSTEMLIB] ERROR : file '$template_name' not found by full path '$template_path'";
		sw_log_warn( $error );
		return $error;
	}

	$args{TYPE} = 'FILE';
	$args{SOURCE} = $template_path;

	my $result;
	my $template = $sys->find_text_template(%args);
	if(ref($input) eq 'HASH' || !$input){
		$input->{custom} = $sys->ui_palette;
		$input->{custom_version} = $ENV{HSPC_CU_SKIN_VERSION};
		#
		$result = $template->fill_in(HASH => $input, BROKEN => \&_sw_web_template_broken,BROKEN_ARG=>\$template_path,%args);
	}else{
		$result = $template->fill_in(PACKAGE  => $input, BROKEN => \&_sw_web_template_broken,BROKEN_ARG=>\$template_path,%args);
	}
	$result=~s/<txt_([^\s>]+(\s+\w+=\"(|.*?[^\\])\")*?)\/?>/_js_escaped_string($1)/ge;
	$result=~s/<env_(\w+)\/?>/$ENV{$1}/ge;
	$result=~s/<img_(\w+)\/?>/$sys->ui_image_src(id=>$1)/ge;
	$result=~s/<imgpath_(\w+)\/?>/$sys->ui_image_path(id=>$1)/ge;## handle tag <imgpath_id>
	$result=~s/<img_(\w+)\s/'<img src = "'.$sys->ui_image_path(id=>$1).'" '/ge;## handle tag <img_id ...>
	$result=~s/<param_(\w+)\/?>/$skin->param($1)/ge;## handle tag <param_id>

	return $result;
}

sub _js_escaped_string {
	my $s = shift;
	$s = string($s);
	$s =~ s/'/\&rsquo;/g;
	return $s;
}##_js_escaped_str

# > source/class_name/object
# > name
# > input
# >= named args for Text::Template->fill_in()
sub sw_web_parse_text_template {
	my $source = shift;
	my $name   = shift;
	my $input  = shift;
	my %args   = (@_);
	sw_die('input not a hashref') unless $input && ref($input) eq 'HASH';

	if (index($source, '::') > -1) {
		$source =~ s#::#/#g;
	}
	$source .= ($source ? '/' : '') . $name
		if defined $name;

	$source = "/var/opt/hspc-root/template/$source";

	if (!-r $source) {
		my $error =
			"[WEBSYSTEMLIB] ERROR : file '$name' not found by full path '$source'";
		sw_log_warn($error);
		return $error;
	}

	$args{TYPE}   = 'FILE';
	$args{SOURCE} = $source;

	my $template = new Text::Template(%args);
	sw_die("Error: template '$args{SOURCE}' is broken: $Text::Template::ERROR")
		unless $template->compile;

	my $result = $template->fill_in(
		HASH       => $input,
		BROKEN     => \&_sw_web_template_broken,
		BROKEN_ARG => \$source,
		%args
	);
	$result =~ s/<txt_(\w+)>/string($1)/ge;

	return $result;
}

# > key
# > format , default = ([^/]+)
# < value
sub sw_uriparam{
	my $key = shift;
	my $c = shift||'([^/]+)';

	my $uri = $ENV{layout}?$ENV{layout}->url:$ENV{PATH_INFO};
	return $2 if $uri =~ m#/$key(,|;)$c(/|$)#;
	return '';
}

# Joint uriparam & sw_argparam - seachs URI first, unless fond, searches CGI parameters
sub sw_param{
	my ($key,$reverse,$no_trim) = @_;

	my $ret;
	if( not $reverse ){
		my $uri = $ENV{layout}?$ENV{layout}->url:$ENV{PATH_INFO};
		if ($uri =~ m#/$key[,;]([^/]+)(/|$)#){
		    $ret = $1;
		} else {
		    $ret = $ENV{q}->param($key);
		}
	}else{
		my $v = $ENV{q}->param($key);
		return $v if defined $v and $v ne '';

		my $uri = $ENV{layout}?$ENV{layout}->url:$ENV{PATH_INFO};
		if ($uri =~ m#/$key[,;]([^/]+)(/|$)#){
			$ret = $1;
		}
	}
	return $no_trim 
		? $ret
		: sw_trim($ret);
}

# < params
sub sw_uriparams{
	my $params;

	my $uri = $ENV{layout}?$ENV{layout}->url:$ENV{PATH_INFO};
	while( $uri =~ m#/(\w+),([^/]+)#g){
		$params->{$1} = $2;
	}
	return $params;
}

# > key  # REQ_Y #
# > type # REQ_N #
# > data # REQ_N # for complex data : ref to hash
# < value
sub sw_argparam{
	my ($key,$type,$data,$no_trim) = @_;
	sw_die('ERROR: you must define key argument sw_argparam()') 
		unless defined $key;

	return undef
		unless $ENV{q};

	$type = $ENV{q}->param("_type_$key") unless defined $type;
	if($type eq 'check'){
		my $value = $ENV{q}->param($key);
		return $value?1:0;
	}elsif($type eq 'datebar'){
		if (sw_argparam("year_$key")){
			return time_local_to_gmt(sw_argparam_datetime(@_));
		} else {
			return undef;
		}
	}elsif($type eq 'date'){
		return sw_argparam_datetime(@_);
	}elsif($type eq 'period'){
		return sw_argparam_period(@_);
	}elsif($type eq 'phone'){
		sw_die('ERROR: you must define data argument for sw_argparam()') unless $data;
		my $prefix = $ENV{q}->param('_dataprefix_'.$key);
		my ($dataitem) = $key =~ /([^_]+)$/;
		$dataitem ||= 'phone';
		$data->{$prefix.$dataitem} = _sw_argparam_phone($key);
		$data->{$prefix.$dataitem.'_src'} = _sw_argparam_phone_src($key);
	}elsif($type eq 'geo_short' || $type eq 'geo_full'){
		sw_die('ERROR: you must define data argument for sw_argparam()') unless $data;
		$data->{state} = sw_argparam($key.'__state');
		$data->{state_alt} = sw_argparam($key.'__state_alt');
		$data->{country} = sw_argparam($key.'__country');
		if($type eq 'geo_full'){
			$data->{address1} = sw_argparam($key.'__address1');
			$data->{address2} = sw_argparam($key.'__address2');
			$data->{house_num} = sw_argparam($key.'__house_num');
			$data->{house_suff} = sw_argparam($key.'__house_suff');
			$data->{zip} = sw_argparam($key.'__zip');
			$data->{city} = sw_argparam($key.'__city');
			sw_argparam($key.'__phone',undef,$data);
			sw_argparam($key.'__mobile',undef,$data);
			sw_argparam($key.'__fax',undef,$data);
		}
		return undef;
	}elsif($type eq 'fio'){
		sw_die('ERROR: you must define data argument for sw_argparam()') unless $data;

		$data->{title} = sw_argparam($key.'__title');
		$data->{first} = sw_argparam($key.'__first');
		$data->{middle} = sw_argparam($key.'__middle');
		$data->{insertion} = sw_argparam($key.'__insertion');
		$data->{last} = sw_argparam($key.'__last');
		$data->{suffix} = sw_argparam($key.'__suffix');

		return undef;
	}elsif($type eq 'fio1'){
		sw_die('ERROR: you must define data argument for sw_argparam()') unless $data;

		my $prefix = $ENV{q}->param('_typedata_'.$key);
		$data->{$prefix.'prefix'} = sw_argparam($key.'__prefix');
		$data->{$prefix.'fname'} = sw_argparam($key.'__fname');
		$data->{$prefix.'mname'} = sw_argparam($key.'__mname');
		$data->{$prefix.'insertion'} = sw_argparam($key.'__insertion');
		$data->{$prefix.'lname'} = sw_argparam($key.'__lname');
		$data->{$prefix.'suffix'} = sw_argparam($key.'__suffix');

		return undef;
	}elsif($type eq 'html'){
		my $value = $ENV{q}->param($key);
		return $value;
	}elsif($type eq 'input'){
		my $value = sw_filter_lt_gt($ENV{q}->param($key));
		unless( $ENV{q}->param('_sys_'.$key.'_allow_trailspaces') ){
			$value=~s|^[ ]*(.*?)[ ]*$||;
			$value=$1;
		}
		return $value;
	} elsif ( $type eq 'lang_input' || $type eq 'lang_textarea' ) {
		my $value;
		require HSPC::MT::GM;
		my $pc = HSPC::MT::GM->find_provider_config(
			id=>$ENV{system_obj}->provider_id()
		) || sw_die("Cannot find provider config #".$ENV{system_obj}->provider_id());
		my $langs = $pc->langs_info();
		if ($ENV{'q'}->param('_no_js')) {
			$value->{$pc->def_lang} =
				$ENV{'q'}->param($key . '_lang'); ## explicitly set default
		} else {
			foreach my $l (@{$langs}) {
				$value->{$l->{id}} = $ENV{'q'}->param($key . '_' . $l->{id});
				if ($type eq 'lang_input') {
					$value->{$l->{id}} = sw_filter_lt_gt($value->{$l->{id}});
					unless ($ENV{'q'}->param('_sys_' . $key . '_allow_trailspaces')) {
						$value->{$l->{id}} =~ s|^[ ]*(.*?)[ ]*$||;
						$value->{$l->{id}} = $1;
					}
				}
			}
		}
		$value->{def} = $value->{$pc->def_lang}; ## store default value
		return $value;
	}elsif($type eq 'list'){
		my @a = $ENV{q}->param($key);
		return \@a;
	}elsif($type eq 'money' || $type eq 'percent' || $type eq 'float') {
		my $value = $ENV{q}->param($key);
		my $vendor_id = $ENV{q}->param("_vendorid_$key");
		if ($vendor_id) {
			require HSPC::MT::Billing::CurrencyTools;
			my $currency_radix =
				HSPC::MT::Billing::CurrencyTools::sw_currency_radix($vendor_id);
			if ($currency_radix && $currency_radix ne '.') {
				$value =~ s/\Q$currency_radix\E/\./;
			}
		}
		return $value;
	} elsif($type eq  'email'){
		my $value = $ENV{q}->param($key);
		return sw_trim($value);
	} elsif ($ENV{q} && $ENV{q}->isa('Apache::Request')) {
		my $upload = $ENV{q}->upload($key);
		return $upload
			if $type eq 'upload';
		return $upload->fh
			if $upload && $upload->filename;
	}

	### handle non-JS clients
	my %trans = (
		'item_submit'=>'btn_submit',
		'item_cancel'=>'btn_cancel',
		'item_delete'=>'btn_delete',
	);
	if ( $ENV{q}->param('__webpage_no_js__') && length($trans{$key}) ){
		$key = $trans{$key};
	}
	return $no_trim 
		? $ENV{q}->param($key)
		: sw_trim($ENV{q}->param($key));
}


# out-of date function, use sw_argparam()
sub sw_argparam_geo{
	return sw_argparam(shift,undef,shift);
}


sub sw_argparam_list {
	return sw_argparam(shift,'list');
}


# > name
# < time
sub sw_argparam_datetime{
	my $n = shift;

	return sprintf("%04d-%02d-%02d %02d:%02d:00",sw_argparam("year_$n"),
		sw_argparam("month_$n"),sw_argparam("day_$n"),sw_argparam("hour_$n"),sw_argparam("min_$n" )
	);
}

# > name
# < period
sub sw_argparam_period{
	my $n = shift;

	return sw_argparam("number_$n") * 31104000 if (sw_argparam("interval_$n") eq 5);
        return sw_argparam("number_$n") * 2592000 if (sw_argparam("interval_$n") eq 4);
        return sw_argparam("number_$n") * 86400 if (sw_argparam("interval_$n") eq 3);
        return sw_argparam("number_$n") * 3600 if (sw_argparam("interval_$n") eq 2);
        return sw_argparam("number_$n") * 60 if (sw_argparam("interval_$n") eq 1);
        return sw_argparam("number_$n");
}

## > name
## < phone
sub _sw_argparam_phone
{
	my ($n) = @_;
	my @res;
	my $chunk = sw_argparam($n."_country_code");
	## $chunk =~ s![\-\+\s]+!!g;
	push @res, "+".$chunk if length($chunk);
	$chunk = sw_argparam($n."_area_code");
	## $chunk =~ s![-\s]+!!g;
	push @res, "(".$chunk.")" if length($chunk);
	$chunk = sw_argparam($n."_number");
	## $chunk =~ s![-\s]+!!g;
	push @res, $chunk if length($chunk);
	$chunk = sw_argparam($n."_extension");
	## $chunk =~ s![-\s]+!!g;
	push @res, $chunk if length($chunk);
	return join(' ',@res);
	## real checks and clearing of delimeters
	## are in MT::Core::Phone
}## argparam_phone


## > name
## < phone
sub _sw_argparam_phone_src
{
	my ($n) = @_;
	my @str;
	for (qw/country_code area_code number extension/){
		my $val = sw_argparam($n.'_'.$_);
		$val =~ s/\&/\&;/g;
		$val =~ s/\|/\&\&/g;
		push @str, length($val) ? $val : '';
	}
	return join('|',@str);
}

# > obj
# > names ( string or strings array)
sub sw_argparams_obj{
	my $o = shift;
	my $control_type;
	my @names = @_;
	my ($c,$c1);
	#
	foreach my $n (@names){
		$control_type = sw_argparam("_type_$n");
		if(sw_argparam("year_$n")){
			$o->{$n} = sw_argparam_datetime($n);
			next;
		}elsif(sw_argparam($n.'__list')){
			$o->{$n} = sw_argparam($n,'list');
			next;
		}elsif(sw_argparam("interval_$n")){
			$o->{$n} = sw_argparam_period($n);
			next;
		}

		$c = sw_argparam($n,undef,$o);
		$o->{$n}=$c if defined $c;
	}
	return $o;
}

## < http/https
sub sw_web_server_protocol{
	my %h = ( tool_id => undef, vendor_id => undef, @_ );
	if($h{tool_id} && $h{vendor_id}) {
		## Tool id provided
		require HSPC::MT::GM::SSLConfig;
		my $ssl_config = HSPC::MT::GM::SSLConfig->new(provider_id=>$h{vendor_id});
		if($ssl_config->provider_id()) {
			my $func = $h{tool_id} . '_has_ssl';
			return $ssl_config->$func() ? 'https' : 'http';
		}
	}
	if (&sw_web_server_port eq $CONFIG{'BG_SERVER_NOSSL_PORT'}) {
		return "http";
	} elsif  (&sw_web_server_port eq $CONFIG{'BG_SERVER_SSL_PORT'}) {
		return "https";
	} elsif($ENV{q}) {
		$ENV{q}->self_url()=~m|^(.+?)://|;
		return lc($1);
	}
	else {
		return 'http';
	}
}

## < virtual hostname
sub sw_web_server_name {
	return $ENV{'SERVER_NAME'};
}

## < port of virtual hostname
sub sw_web_server_port {
	return $ENV{'SERVER_PORT'};
}


sub _sw_web_template_broken {
	my %args = @_;
	my $ref_arg = $args{arg};
	my $c = "[WEBSYSTEMLIB] ERROR : template '$$ref_arg' broken at line number $args{lineno}: $args{error}";
	sw_log_error $c;
 	return qq|<H3><font color="red">Template broken</font></H3><!-- $args{error} --></font></H3>|;
}

# returns full name of the person, eliminates unnecessary space characters
sub sw_full_name {
	my $self = shift;
	return join ' ', grep { $_ } map { $self->{$_} } qw (name_title name_first name_middle insertion name_last name_suffix) if $self->{name_last};
	return join ' ', grep { $_ } map { $self->{$_} } qw (prefix fname mname insertion lname suffix) if $self->{lname};
}

sub sw_sec_env {
    my %result;

    $result{added_by} = $ENV{'security_obj'}->person_id;
    $result{added_by_account} = $ENV{'security_obj'}->account_no;
    $result{added_by_ip} = $ENV{REMOTE_ADDR};

    return \%result;
}

## replaces special HTML symbols with their codes
## e.g. '<' -> '&#60;'
## argument hash:
##	>>		text 		- string to encode,
##	=>		convert_ln 	- either convert \n (\r\n) to <br> or not
## returns encoded string
sub sw_web_prepare4browser{
	my %args = @_;
	my $val = 0;
	my $text = $args{'text'} or
		return '';
	## define characters' codes to be encoded
	my @codes = (34, 60, 62, 338, 339, 352, 353, 376, 710,
		732, 8194, 8195, 8201, 8204, 8205, 8206, 8207, 8211, 8212,
		8216, 8217, 8218, 8220, 8221, 8222, 8224, 8225, 8240, 8249,
		8250, 8364);

	## first of all - to avoid replace of '&' in already placed entities:
	$text =~ s/\&/&#38;/g; 	## replace '&' symbol

	foreach my $code (@codes){
		$val = chr($code);
		$text =~ s/$val/&#$code;/g; ## replace symbols with their HTML entities
	}
	## replace \n and \r\n to <br>
	$text =~ s/\r*\n/<br>/g if ($args{'convert_ln'});
	return $text;
}

## just filter '<' and '>'
## takes string(s)
## returns filtered string(s)
sub sw_filter_lt_gt {
	my @args = @_;
	for (@args) {
		s/[<>]//g;
	}
	return wantarray ? @args : $args[0];
}


## > provider_id
## < [['en','English'],...] first is default
sub sw_sorted_lang_options {
	my $provider_id = shift || sw_die(
		"sw_sorted_lang_options: \$provider_id is required."
	);
	require HSPC::MT::GM;
	my $config = HSPC::MT::GM->find_provider_config(id=>$provider_id) || sw_die(
		"sw_sorted_lang_options: provider config not found for #$provider_id"
	);
	my (@langs,$lh);
	foreach my $l ( @{$config->langs()} ) {
		next if ( $l->{lang} eq $config->def_lang() );
		$lh = HSPC::Localization::get_language($l->{lang});
		push @langs,[$lh->{id},$lh->{title_id}];
	}
	$lh = HSPC::Localization::get_language($config->def_lang());
	unshift @langs,[$lh->{id},$lh->{title_id}] if ( $lh->{id} );
	unshift @langs,['','use default'] unless ( scalar(@langs) );
	return \@langs;
}##/sw_sorted_lang_options


sub sw_provider_def_lang {
	my $provider_id = shift || sw_die(
		"sw_provider_def_lang: \$provider_id is required."
	);
	require HSPC::MT::GM;
	my $config = HSPC::MT::GM->find_provider_config(id=>$provider_id) || sw_die(
		"sw_provider_def_lang: provider config not found for #$provider_id"
	);
	return $config->def_lang();
}##/sw_provider_def_lang


1;

__END__
